import { OnInit, Signal } from "@angular/core";
import { ISdSortingDef } from "../../utils/managers/sd-sorting-manager";
import { TSdViewType } from "../../utils/signals/use-view-type.signal";
import { SdDataSheetColumnDirective } from "./sd-data-sheet-column.directive";
import { SdDataSheetFilterDirective } from "./sd-data-sheet-filter.directive";
import { SdDataSheetToolDirective } from "./sd-data-sheet-tool.directive";
import { ISdSelectModal, ISelectModalOutputResult } from "./sd-data-select-button.control";
import * as i0 from "@angular/core";
export declare class SdDataSheetControl {
    protected readonly icons: import("../../providers/sd-angular-config.provider").ISdAngularIcon;
    parent: any;
    insertText: import("@angular/core").InputSignal<string>;
    deleteText: import("@angular/core").InputSignal<string>;
    filterControls: Signal<readonly SdDataSheetFilterDirective[]>;
    toolControls: Signal<readonly SdDataSheetToolDirective[]>;
    columnControls: Signal<readonly SdDataSheetColumnDirective<any>[]>;
    beforeToolControls: Signal<SdDataSheetToolDirective[]>;
    afterToolControls: Signal<SdDataSheetToolDirective[]>;
    onFilterSubmit(): void;
    onRefreshCommand(): void;
    onCreateItemButtonClick(): Promise<void>;
    onEditItemButtonClick(item: any, index: number, event: MouseEvent): Promise<void>;
    onToggleDeleteItemsButtonClick(del: boolean): Promise<void>;
    onDownloadExcelButtonClick(): Promise<void>;
    onUploadExcelButtonClick(): Promise<void>;
    onConfirmButtonClick(): Promise<void>;
    onCancelButtonClick(): Promise<void>;
    static ɵfac: i0.ɵɵFactoryDeclaration<SdDataSheetControl, never>;
    static ɵcmp: i0.ɵɵComponentDeclaration<SdDataSheetControl, "sd-data-sheet", never, { "insertText": { "alias": "insertText"; "required": false; "isSignal": true; }; "deleteText": { "alias": "deleteText"; "required": false; "isSignal": true; }; }, {}, ["filterControls", "toolControls", "columnControls"], never, true, never>;
}
export declare abstract class AbsSdDataSheet<F extends Record<string, any>, I, K> implements ISdSelectModal, OnInit {
    #private;
    restricted?: Signal<boolean>;
    readonly?: Signal<boolean>;
    abstract key: string;
    initFilter?(): F;
    abstract getItemInfo(item: I): ISdDataSheetItemInfo<K>;
    abstract search(param: TSdDataSheetSearchParam<F>): Promise<ISdDataSheetSearchResult<I>>;
    editItem?(item?: I): Promise<boolean | undefined>;
    toggleDeleteItems?(selectedItems: I[], del: boolean): Promise<boolean>;
    downloadExcel?(items: I[]): Promise<void>;
    uploadExcel?(file: File): Promise<void>;
    viewType: import("@angular/core").InputSignal<TSdViewType | undefined>;
    currViewType: Signal<TSdViewType>;
    busyCount: import("../../utils/bindings/$signal").SdWritableSignal<number>;
    initialized: import("../../utils/bindings/$signal").SdWritableSignal<boolean>;
    close: import("@angular/core").OutputEmitterRef<ISelectModalOutputResult>;
    selectMode: import("@angular/core").InputSignal<"single" | "multi" | undefined>;
    selectedItemKeys: import("@angular/core").ModelSignal<K[]>;
    items: import("../../utils/bindings/$signal").SdWritableSignal<I[]>;
    summaryData: import("../../utils/bindings/$signal").SdWritableSignal<Partial<I>>;
    selectedItems: import("../../utils/bindings/$signal").SdWritableSignal<I[]>;
    trackByFn: (item: I) => K;
    page: import("../../utils/bindings/$signal").SdWritableSignal<number>;
    pageLength: import("../../utils/bindings/$signal").SdWritableSignal<number>;
    sortingDefs: import("../../utils/bindings/$signal").SdWritableSignal<ISdSortingDef[]>;
    filter: import("../../utils/bindings/$signal").SdWritableSignal<F>;
    lastFilter: import("../../utils/bindings/$signal").SdWritableSignal<F>;
    ngOnInit(): void;
    constructor();
    doFilterSubmit(): void;
    doRefresh(): void;
    refresh(): Promise<void>;
    doEditItem(item?: I): Promise<void>;
    isSelectedItemsHasDeleted: Signal<boolean>;
    isSelectedItemsHasNotDeleted: Signal<boolean>;
    getItemCellStyleFn: (item: I) => "text-decoration: line-through;" | undefined;
    doToggleDeleteItems(del: boolean): Promise<void>;
    doDownloadExcel(): Promise<void>;
    doUploadExcel(): Promise<void>;
    doModalConfirm(): void;
    doModalCancel(): void;
    static ɵfac: i0.ɵɵFactoryDeclaration<AbsSdDataSheet<any, any, any>, never>;
    static ɵdir: i0.ɵɵDirectiveDeclaration<AbsSdDataSheet<any, any, any>, never, never, { "viewType": { "alias": "viewType"; "required": false; "isSignal": true; }; "selectMode": { "alias": "selectMode"; "required": false; "isSignal": true; }; "selectedItemKeys": { "alias": "selectedItemKeys"; "required": false; "isSignal": true; }; }, { "close": "close"; "selectedItemKeys": "selectedItemKeysChange"; }, never, never, true, never>;
}
export interface ISdDataSheetItemInfo<K> {
    key: K;
    isDeleted?: boolean;
}
export type TSdDataSheetSearchParam<F extends Record<string, any>> = ISdDataSheetSearchExcelParam<F> | ISdDataSheetSearchSheetParam<F>;
interface ISdDataSheetSearchExcelParam<F extends Record<string, any>> {
    type: "excel";
    lastFilter: F;
    sortingDefs: ISdSortingDef[];
}
interface ISdDataSheetSearchSheetParam<F extends Record<string, any>> {
    type: "sheet";
    lastFilter: F;
    sortingDefs: ISdSortingDef[];
    page: number;
}
export interface ISdDataSheetSearchResult<I> {
    items: I[];
    pageLength?: number;
    summary?: Partial<I>;
}
export {};
