import { ChangeDetectionStrategy, Component, ElementRef, output, viewChild, ViewEncapsulation, } from "@angular/core";
import { SdBusyContainerControl } from "../controls/sd-busy-container.control";
import { $signal } from "../utils/bindings/$signal";
import * as i0 from "@angular/core";
const _c0 = ["content"];
export class SdAddressSearchModal {
    constructor() {
        this.contentElRef = viewChild.required("content", {
            read: ElementRef,
        });
        this.close = output();
        this.initialized = $signal(false);
    }
    async ngOnInit() {
        if (!document.getElementById("daum_address")) {
            await new Promise((resolve) => {
                const scriptEl = document.createElement("script");
                // scriptEl.async = true;
                scriptEl.src = "//t1.daumcdn.net/mapjsapi/bundle/postcode/prod/postcode.v2.js";
                scriptEl.setAttribute("id", "daum_address");
                scriptEl.onload = () => {
                    // @ts-expect-error
                    daum.postcode.load(() => {
                        resolve();
                    });
                };
                document.head.appendChild(scriptEl);
            });
        }
        const contentEl = this.contentElRef().nativeElement;
        // @ts-expect-error
        new daum.Postcode({
            oncomplete: (data) => {
                const addr = data.userSelectedType === "R" ? data.roadAddress : data.jibunAddress;
                let extraAddr = "";
                if (data.userSelectedType === "R") {
                    if (data.bname !== "" && /[동로가]$/g.test(data.bname)) {
                        extraAddr += data.bname;
                    }
                    if (data.buildingName !== "" && data.apartment === "Y") {
                        extraAddr += extraAddr !== "" ? ", " + data.buildingName : data.buildingName;
                    }
                    if (extraAddr !== "") {
                        extraAddr = " (" + extraAddr + ")";
                    }
                }
                this.close.emit({
                    postNumber: data.zonecode,
                    address: addr + extraAddr,
                    buildingName: data.buildingName,
                });
            },
            onresize: (size) => {
                contentEl.style.height = size.height + "px";
            },
            width: "100%",
            height: "100%",
        }).embed(contentEl, { autoClose: false });
        this.initialized.set(true);
    }
    static { this.ɵfac = function SdAddressSearchModal_Factory(__ngFactoryType__) { return new (__ngFactoryType__ || SdAddressSearchModal)(); }; }
    static { this.ɵcmp = /*@__PURE__*/ i0.ɵɵdefineComponent({ type: SdAddressSearchModal, selectors: [["sd-address-search-modal"]], viewQuery: function SdAddressSearchModal_Query(rf, ctx) { if (rf & 1) {
            i0.ɵɵviewQuerySignal(ctx.contentElRef, _c0, 5, ElementRef);
        } if (rf & 2) {
            i0.ɵɵqueryAdvance();
        } }, outputs: { close: "close" }, decls: 3, vars: 1, consts: [["content", ""], [3, "busy"], [2, "min-height", "100px"]], template: function SdAddressSearchModal_Template(rf, ctx) { if (rf & 1) {
            i0.ɵɵelementStart(0, "sd-busy-container", 1);
            i0.ɵɵelement(1, "div", 2, 0);
            i0.ɵɵelementEnd();
        } if (rf & 2) {
            i0.ɵɵproperty("busy", !ctx.initialized());
        } }, dependencies: [SdBusyContainerControl], encapsulation: 2, changeDetection: 0 }); }
}
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SdAddressSearchModal, [{
        type: Component,
        args: [{
                selector: "sd-address-search-modal",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                standalone: true,
                imports: [SdBusyContainerControl],
                template: `
    <sd-busy-container [busy]="!initialized()">
      <div #content style="min-height: 100px;"></div>
    </sd-busy-container>
  `,
            }]
    }], null, null); })();
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassDebugInfo(SdAddressSearchModal, { className: "SdAddressSearchModal", filePath: "packages/sd-angular/src/modals/sd-address-search.modal.ts", lineNumber: 26 }); })();
//# sourceMappingURL=sd-address-search.modal.js.map