import { Signal } from "@angular/core";
import { IconDefinition } from "@fortawesome/fontawesome-svg-core";
import * as i0 from "@angular/core";
export declare function usePermsSignal<K extends string>(viewCodes: string[], keys: K[]): Signal<K[]>;
export declare abstract class SdAppStructureProvider<TModule = unknown> {
    abstract items: TSdAppStructureItem<TModule>[];
    abstract usableModules: Signal<TModule[] | undefined>;
    abstract permRecord: Signal<Record<string, boolean> | undefined>;
    usableMenus: Signal<ISdMenu<TModule>[]>;
    usableFlatMenus: Signal<ISdFlatMenu<TModule>[]>;
    getPermissionsByStructure(items: TSdAppStructureItem<TModule>[], codeChain?: string[]): ISdPermission<TModule>[];
    getTitleByFullCode(fullCode: string): string;
    getPermsByFullCode<K extends string>(fullCodes: string[], permKeys: K[]): K[];
    static ɵfac: i0.ɵɵFactoryDeclaration<SdAppStructureProvider<any>, never>;
    static ɵprov: i0.ɵɵInjectableDeclaration<SdAppStructureProvider<any>>;
}
export declare abstract class SdAppStructureUtils {
    #private;
    static getTitleByFullCode<TModule>(items: TSdAppStructureItem<TModule>[], fullCode: string): string;
    static getPermsByFullCode<TModule, K extends string>(items: TSdAppStructureItem<TModule>[], fullCodes: string[], permKeys: K[], permRecord: Record<string, boolean> | undefined): K[];
    static getMenus<TModule>(items: TSdAppStructureItem<TModule>[], codeChain: string[], usableModules: TModule[] | undefined, permRecord: Record<string, boolean> | undefined): ISdMenu<TModule>[];
    static getFlatMenus<TModule>(items: TSdAppStructureItem<TModule>[], usableModules: TModule[] | undefined, permRecord: Record<string, boolean> | undefined): ISdFlatMenu<TModule>[];
    static getPermissions<TModule>(items: TSdAppStructureItem<TModule>[], codeChain: string[], usableModules: TModule[] | undefined): ISdPermission<TModule>[];
    static getFlatPermissions<TModule>(items: TSdAppStructureItem<TModule>[], usableModules: TModule[] | undefined): ISdFlatPermission<TModule>[];
}
export type TSdAppStructureItem<TModule = unknown> = ISdAppStructureGroupItem<TModule> | ISdAppStructureLeafItem<TModule>;
interface ISdAppStructureGroupItem<TModule> {
    code: string;
    title: string;
    modules?: TModule[];
    icon?: IconDefinition;
    children: TSdAppStructureItem<TModule>[];
}
interface ISdAppStructureLeafItem<TModule> {
    code: string;
    title: string;
    modules?: TModule[];
    perms?: ("use" | "edit")[];
    subPerms?: ISdAppStructureSubPermission<TModule>[];
    icon?: IconDefinition;
    isNotMenu?: boolean;
}
interface ISdAppStructureSubPermission<TModule> {
    code: string;
    title: string;
    modules?: TModule[];
    perms: ("use" | "edit")[];
}
export interface ISdMenu<TModule = unknown> {
    title: string;
    codeChain: string[];
    icon: IconDefinition | undefined;
    modules: TModule[] | undefined;
    children: ISdMenu<TModule>[] | undefined;
}
export interface ISdFlatMenu<TModule = unknown> {
    titleChain: string[];
    codeChain: string[];
    modulesChain: TModule[][];
}
export interface ISdPermission<TModule = unknown> {
    title: string;
    codeChain: string[];
    modules: TModule[] | undefined;
    perms: ("use" | "edit")[] | undefined;
    children: ISdPermission<TModule>[] | undefined;
}
export interface ISdFlatPermission<TModule = unknown> {
    titleChain: string[];
    codeChain: string[];
    modulesChain: TModule[][];
}
export {};
