import { ApplicationRef, createComponent, EventEmitter, inject, Injectable, Injector, inputBinding, outputBinding, } from "@angular/core";
import { SdModalControl } from "../controls/sd-modal.control";
import { $signal } from "../utils/bindings/$signal";
import { Wait } from "@simplysm/sd-core-common";
import { SdBusyProvider } from "./sd-busy.provider";
import * as i0 from "@angular/core";
export class SdModalInstance {
    #activatedModalProvider;
    #compRef;
    #modalRef;
    #prevActiveEl;
    #open;
    constructor(appRef, modal, options) {
        this.#open = $signal(false);
        this.close = new EventEmitter();
        //-- busy
        const sdBusy = appRef.injector.get(SdBusyProvider);
        sdBusy.globalBusyCount.update((v) => v + 1);
        //-- new Provider
        this.#activatedModalProvider = new SdActivatedModalProvider();
        //-- Content component
        this.#compRef = createComponent(modal.type, {
            environmentInjector: appRef.injector,
            elementInjector: Injector.create({
                parent: appRef.injector,
                providers: [{ provide: SdActivatedModalProvider, useValue: this.#activatedModalProvider }],
            }),
            bindings: [
                ...Object.keys(modal.inputs).map((inputKey) => inputBinding(inputKey, () => modal.inputs[inputKey])),
                outputBinding("close", (val) => this.#onComponentClosed(val)),
            ],
        });
        this.#activatedModalProvider.contentComponent.set(this.#compRef.instance);
        const compEl = this.#compRef.location.nativeElement;
        //-- Modal component
        this.#modalRef = createComponent(SdModalControl, {
            environmentInjector: appRef.injector,
            projectableNodes: [[compEl]],
            elementInjector: Injector.create({
                parent: appRef.injector,
                providers: [{ provide: SdActivatedModalProvider, useValue: this.#activatedModalProvider }],
            }),
            bindings: [
                inputBinding("title", () => modal.title),
                inputBinding("key", () => options?.key),
                inputBinding("hideHeader", () => options?.hideHeader ?? false),
                inputBinding("hideCloseButton", () => options?.hideCloseButton ?? false),
                inputBinding("useCloseByBackdrop", () => options?.useCloseByBackdrop ?? false),
                inputBinding("useCloseByEscapeKey", () => options?.useCloseByEscapeKey ?? false),
                inputBinding("float", () => options?.float ?? false),
                inputBinding("minHeightPx", () => options?.minHeightPx),
                inputBinding("minWidthPx", () => options?.minWidthPx),
                inputBinding("resizable", () => options?.resizable ?? true),
                inputBinding("movable", () => options?.movable ?? true),
                inputBinding("headerStyle", () => options?.headerStyle),
                inputBinding("fill", () => options?.fill ?? false),
                inputBinding("open", this.#open),
                outputBinding("openChange", (val) => this.#onModalOpenChange(val)),
            ],
        });
        this.#activatedModalProvider.modalComponent.set(this.#modalRef.instance);
        const modalEl = this.#modalRef.location.nativeElement;
        this.#prevActiveEl = document.activeElement;
        document.body.appendChild(modalEl);
        appRef.attachView(this.#compRef.hostView);
        appRef.attachView(this.#modalRef.hostView);
        requestAnimationFrame(async () => {
            if (!this.#compRef.instance.initialized()) {
                await Wait.until(() => this.#compRef.instance.initialized(), undefined, 60 * 1000);
            }
            setTimeout(() => {
                sdBusy.globalBusyCount.update((v) => v - 1);
                this.#open.set(true);
                requestAnimationFrame(() => {
                    if (options?.noFirstControlFocusing) {
                        this.#modalRef.instance.dialogElRef().nativeElement.focus();
                    }
                    else {
                        (compEl.findFocusableFirst() ?? this.#modalRef.instance.dialogElRef().nativeElement).focus();
                    }
                });
            }, 100);
        });
    }
    #onModalOpenChange(val) {
        if (!this.#activatedModalProvider.canDeactivefn())
            return;
        if (val) {
            this.#open.set(val);
        }
        else {
            this.#onComponentClosed(undefined);
        }
    }
    #onComponentClosed(val) {
        if (val != null && !this.#activatedModalProvider.canDeactivefn())
            return;
        const modalEl = this.#modalRef.location.nativeElement;
        modalEl.addEventListener("transitionend", () => {
            this.#compRef.destroy();
            this.#modalRef.destroy();
        });
        this.#open.set(false);
        if (this.#prevActiveEl) {
            this.#prevActiveEl.focus();
        }
        this.close.emit(val);
    }
}
export class SdModalProvider {
    constructor() {
        this.#appRef = inject(ApplicationRef);
        this.modalCount = $signal(0);
    }
    #appRef;
    async showAsync(modal, options) {
        return await new Promise((resolve, reject) => {
            this.modalCount.update((v) => v + 1);
            try {
                const modalInstance = new SdModalInstance(this.#appRef, modal, options);
                modalInstance.close.subscribe((val) => {
                    resolve(val);
                });
            }
            catch (err) {
                reject(err);
            }
            this.modalCount.update((v) => v - 1);
        });
    }
    static { this.ɵfac = function SdModalProvider_Factory(__ngFactoryType__) { return new (__ngFactoryType__ || SdModalProvider)(); }; }
    static { this.ɵprov = /*@__PURE__*/ i0.ɵɵdefineInjectable({ token: SdModalProvider, factory: SdModalProvider.ɵfac, providedIn: "root" }); }
}
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SdModalProvider, [{
        type: Injectable,
        args: [{ providedIn: "root" }]
    }], null, null); })();
export class SdActivatedModalProvider {
    constructor() {
        this.modalComponent = $signal();
        this.contentComponent = $signal();
        this.canDeactivefn = () => true;
    }
    static { this.ɵfac = function SdActivatedModalProvider_Factory(__ngFactoryType__) { return new (__ngFactoryType__ || SdActivatedModalProvider)(); }; }
    static { this.ɵprov = /*@__PURE__*/ i0.ɵɵdefineInjectable({ token: SdActivatedModalProvider, factory: SdActivatedModalProvider.ɵfac }); }
}
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SdActivatedModalProvider, [{
        type: Injectable
    }], null, null); })();
//# sourceMappingURL=sd-modal.provider.js.map